/* ------------------------------------------------------------------------
System       : cc65 cross-development environment
Platform     : Apple IIgs or Apple IIe with VOC or CB

Bill Buckels
July 2014

Licence      : You may use this program for whatever you wish as long
			   as you agree that Bill Buckels has no warranty or
			   liability obligations whatsoever from said use.
------------------------------------------------------------------------ */

#ifndef __APPLE2__
#define __APPLE2__
#endif

#include <stdio.h>
#include <string.h>
#include <conio.h>

/* all the core routines etc. are included in the local headers */
#include "shrworld.h"


/* line drawings script demo routines */
int colors[] = {
LOLTBLUE,LOPINK,LOLTGREEN,LOYELLOW,LOAQUA,LOWHITE,-1};

void drawmap(int mapidx,int drawcolor)
{

  int *ptr, xidx=0,yidx=1,newx,oldx,newy,oldy,color,cidx=0;

  /* select vector map or vector object */
  switch(mapidx)
  {
	  case MAPLELEAF:	ptr = (int *)&maple[0];
	  					break;
	  case USAMAP:		ptr = (int *)&usa[0];
	                    break;
	  case USASTATES:   ptr = (int *)&states[0];
	  					break;
	  case WORLDMAP:
	  default:    		ptr = (int *)&world[0];
  }

  /* draw selected map or object */
  for (;;) {

      /* alternating color option */
      if (drawcolor < 0 || drawcolor > 15) {
	  		if (colors[cidx] < 0)cidx = 1;
	  		color = colors[cidx];
	  		cidx++;
	  }
	  else {
		  color = drawcolor;
	  }

	  oldx= ptr[xidx];
	  oldy= ptr[yidx];

	  for(;;)
	  {
		  xidx +=2;
		  yidx +=2;
		  newx= ptr[xidx];
		  newy= ptr[yidx];

		  if (newx == -1) break;

		  /* draw next line */
		  line320(oldx,oldy,newx,newy,color);

		  oldx=newx;
		  oldy=newy;
	   }
	   if (newy == -2)break;
	   xidx +=2;
	   yidx +=2;

   }
}


char *caption1[] = {
"Introducing The Tom Thumb Font",
"",
"The Tom Thumb Font was originally developed as a Palm Pilot font by",
"developer Brian Swetland, and fine-tuned by Robey Pointer. This 4 x 6 fixed-",
"pitch font provides 96 reasonably legible ASCII characters and can be pixel-",
"mapped (or bitmapped) to provide the various displays of the Apple II with a",
"color (or monochrome) font:",
NULL};

char *caption2[] = {
"When the Tom Thumb Font is used on the Super Hi-Res (SHR) Mode320 display, up",
"to 33 rows of 80 column text can be displayed in 4096 possible colors, with",
"different colored characters and foreground and background colors.",
"",
"The Tom Thumb character map used in this demo was entirely hand-made in a",
"programmer's editor by looking at a bitmap of the font and counting pixels.",
NULL};

char *fox ="The Quick Brown Fox jumps over the Lazy Dog!";

void TinyFontDemo()
{
	int i;
	unsigned y=0;
	char buf[33];

	clear320((unsigned char)LODKBLUE);

	/* display top caption */
	for (i=0;caption1[i] != NULL;i++) {
		thumb320(caption1[i],0,y,(unsigned char)LOWHITE,255,'L');
		y+=6;
	}

	/* display font characters */
	buf[32] = 0;
	y+=6;
	for (i=0;i<32;i++)buf[i] = (char)i+32;
	tinyplots(buf,160,y,LOBLACK,LOWHITE,'M');
	y+=6;
	for (i=0;i<32;i++)buf[i] = (char)i+64;
	tinyplots(buf,160,y,LOBLACK,LOWHITE,'M');
	y+=6;
	for (i=0;i<32;i++)buf[i] = (char)i+96;
	tinyplots(buf,160,y,LOBLACK,LOWHITE,'M');
	y+=6;

    y+=6;
    for (i=LORED;i<MAXCOLORS;i++) {
		if (i == LODKBLUE) continue;
		tinyplots(fox,160,y,(unsigned char)i,LOBLACK,'M');
		y+=6;
	}

	/* display bottom caption */
	y+=6;
	for (i=0;caption2[i] != NULL;i++) {
		thumb320(caption2[i],0,y,(unsigned char)LOLTBLUE,255,'L');
		y+=6;
	}

	while (kbhit() > 0)cgetc();
	cgetc();


}


/* kaleidoscope demo helper functions */
void rodgi_setcolor(unsigned color)
{
	/* set a buffer of pixel-pairs for block-drawing */
	memset((char *)&scanline[0],(unsigned char)(color << 4 | color),4);
}

void rodgi_setcenter()
{
	int y;

    /* draw the center block for the demo */
	for (y = 0; y < 5; y++) putpairs320(156,98+y,4,255);
}

void rodgi_setpixel(unsigned x, unsigned y)
{
	unsigned y2;

    /* draw the blocks for the demo */
    /* the demo was orginally written for single pixels in lgr */
	/* convert to shr from lgr */
	/* 40 x 40 to 320 x 200 = 8 pixel x 5 line blocks of 4 bytes */

	x = x * 8;
	y = y * 5;
	y2 = y + 5;

    /* an offset adjustment of 4 pixels and 2 scanlines is
       needed to center the kaleidoscope */
	while (y < y2) {
		putpairs320(x-4,y-2,4,255);
		y++;
	}

}

/* kaleidoscope demo */
int shrod()
{
	unsigned i, j, k, w, fmi, fmk, color;

    clear320(0);

	for (w = 3; w < 51; ++w) {
		for (i = 1; i < 20; ++i) {
			for (j = 0; j < 20; ++j) {
				k = i + j;
				color = (j * 3) / (i + 3) + i * w / 12;
				rodgi_setcolor(color & 0x0f);
				fmi = 40 - i;  fmk = 40 - k;
				rodgi_setpixel(i, k);     rodgi_setpixel(k, i);
				rodgi_setpixel(fmi, fmk); rodgi_setpixel(fmk, fmi);
				rodgi_setpixel(k, fmi);   rodgi_setpixel(fmi, k);
				rodgi_setpixel(i, fmk);   rodgi_setpixel(fmk, i);
				if (kbhit() > 0) {
					return (int) cgetc();
				}
			}
		}
		rodgi_setcenter();
	}

	return 0;
}


int main(void)
{
	int idx,c,x1,x2,y1,y2;
	unsigned char *ptr, ch;

    /* initialize text mode. stay in 40 column mode. */
    texton();
    clrscr();

    /* initialize empty palette and scb's */
    initbuffers();
    puts("Select a 16-color palette:");
    puts("1 - Kegs32");
    puts("2 - CiderPress");
    puts("3 - Old AppleWin");
    puts("4 - New AppleWin");

    ch = cgetc();
    clrscr();

	/* turn shr on */
	shgron();
	/* now that the shr display is in auxiliary memory */
	/* clear the palette and point all the scbs to the
	   first palette */
    clearpalette();
    clearscbs();
	/* clear the shr screen to blue before setting the palette...
	   it will clear to black initially */
    clear320((unsigned char)LODKBLUE);
    /* set the palette that the user has selected
       and the screen will change to blue */
    switch(ch) {
		case '1': ptr = (unsigned char *)&rgbkegs32[0];break;
		case '2': ptr = (unsigned char *)&rgbciderpress[0];break;
		case '3': ptr = (unsigned char *)&rgbawinold[0];break;
		default:  ptr = (unsigned char *)&rgbawinnew[0];break;
	}
	setpalette(ptr,0);
	thumb320("The World of SHR Pixel Graphics in cc65",160,0,LOWHITE,255,'M');
	thumb320("A Demo by Bill Buckels - July 2014",160,194,LOWHITE,255,'M');
    thumb320("Press any key...",160,100,(unsigned char)LOYELLOW,255,'M');
	while (kbhit() > 0)cgetc();
    cgetc();

	clear320(0);
	/* print demo title */
	thumb320("The World of SHR Pixel Graphics in cc65",160,0,LOWHITE,255,'M');
	thumb320("A Demo by Bill Buckels - July 2014",160,194,LOWHITE,255,'M');

    /* circle demo */
	for (idx = LORED; idx < MAXCOLORS; idx++) {
		c = (idx * 4);
		circle320(160,99, 96-c, (unsigned char)idx);
	}
	while (kbhit() > 0)cgetc();
	thumb320("Press any key...",160,99,LOWHITE,255,'M');
	cgetc();

	clear320(0);
	/* print demo title */
	tinyplots("The World of SHR Pixel Graphics in cc65",160,0,LOWHITE,255,'M');
	tinyplots("A Demo by Bill Buckels - July 2014",160,194,LOWHITE,255,'M');

	/* filled circle demo */
	for (idx = 1; idx < 16; idx++) {
		c = (idx * 4);
		disk320(160,99, 96-c, (unsigned char)(MAXCOLORS - idx));
	}
	while (kbhit() > 0)cgetc();
	tinyplots("Press any key...",160,99,LOWHITE,255,'M');
	cgetc();

	clear320(0);
	/* print demo title */
	tinyplots("The World of SHR Pixel Graphics in cc65",160,0,LOWHITE,255,'M');
	tinyplots("A Demo by Bill Buckels - July 2014",160,194,LOWHITE,255,'M');

	/* box demo */
	y1 = 7;
	y2 = 192;
	x1 = 60;
	x2 = 260;
	ch = 255;
	for (idx = LORED; idx < MAXCOLORS; idx++) {
	   c = (idx * 2);
	   if (idx == LOWHITE) ch = LODKBLUE; /* set fillcolor for white box */
	   box320(x1+c,y1+c,x2-c,y2-c,(unsigned char)idx,ch);
	}
	while (kbhit() > 0)cgetc();
	tinyplots("Press any key...",160,99,LOWHITE,255,'M');
	cgetc();

	/* erase text */
	tinyplots("Press another key...",160,99,LOWHITE,LODKBLUE,'M');
	/* candy frame demo */
	ch = 1;
	for (;;) {
		ch = candyframe320(ch);
		if (kbhit() > 0) break;
	}
	while (kbhit() > 0)cgetc();

    /* Tom Thumb Font screen */
	TinyFontDemo();

   	/* first line script screen */
   	clear320(0);
	drawmap(USAMAP,LOMEDBLUE);
	drawmap(USASTATES,-1);
	thumb320("Now you've seen the Colorful State of SHR Pixel Graphics in cc65.",160,0,LOWHITE,255,'M');
	thumb320("Press any key to see the Wonderful World of SHR Pixel Graphics in cc65!",160,194,LOWHITE,255,'M');
	while (kbhit() > 0)cgetc();
    cgetc();

    /* second line script screen */
    clear320((unsigned char)LODKBLUE);
  	drawmap(WORLDMAP,LOPINK);
	thumb320("The World of SHR Pixel Graphics in cc65",160,0,LOWHITE,255,'M');
	thumb320("A Demo by Bill Buckels - July 2014",160,194,LOWHITE,255,'M');
    thumb320("Press any key...",160,100,(unsigned char)LOYELLOW,255,'M');
  	while (kbhit() > 0)cgetc();
  	cgetc();

    /* third line script screen */
    clear320(0);
 	drawmap(MAPLELEAF,LOPINK);
 	tinyplots("Greetings from the Great White North",160,0,LOWHITE,255,'M');
    tinyplots("MADE IN CANADA",160,100,LOYELLOW,255,'M');
    tinyplots("And brought to you by the Letter \"C\"",160,194,LOWHITE,255,'M');
    while (kbhit() > 0)cgetc();
    cgetc();

    /* kaleidoscope demo */
	while (shrod()!=27);

	/* clear the palette and scbs then
	   turn shroff and re-initialize text mode */
	clearpalette();
	clearscbs();
	shgroff();
	texton();
	clrscr();

    /* and exit */
    return 0;


}
