/* ------------------------------------------------------------------------
System       : cc65 cross-development environment
Platform     : Apple IIgs or Apple IIe with VOC or CB

Bill Buckels
July 2014

Licence      : You may use this program for whatever you wish as long
			   as you agree that Bill Buckels has no warranty or
			   liability obligations whatsoever from said use.
------------------------------------------------------------------------ */
#include "shr.h"
#include "tomthumb.h"
#include "linetables.h"

/* lo-res colors and color order */
/* these are also the double lo-res and double hi-res colors */
#define LOBLACK   	0
#define LORED     	1
#define LODKBLUE 	2
#define LOPURPLE  	3
#define LODKGREEN	4
#define LOGRAY    	5
#define LOMEDBLUE	6
#define LOLTBLUE 	7
#define LOBROWN   	8
#define LOORANGE  	9
#define LOGREY    	10
#define LOPINK    	11
#define LOLTGREEN	12
#define LOYELLOW  	13
#define LOAQUA    	14
#define LOWHITE   	15
#define MAXCOLORS   16

/* 4 - palette options */
/* these are the rgb values of the lo-res colors from
   the AppleWin and Kegs32 Emulators and from
   the CiderPress File Viewer.

   two sets of AppleWin Colors are provided:
   one from an older version and one from a newer version.

   the routines in this demo are based on a fixed palette
   so it was convenient to use known colors and color order.


*/
unsigned char rgbawinold[48] = {
	0,0,0,       /* black    */
	208,0,48,	 /* red      */
	0,0,128,	 /* dk blue  */
	255,0,255,	 /* purple   */
	0,128,0,	 /* dk green */
	128,128,128, /* gray     */
	0,0,255,	 /* med blue */
	96,160,255,	 /* lt blue  */
	128,80,0,	 /* brown    */
    255,128,0,	 /* orange   */
    192,192,192, /* grey     */
    255,144,128, /* pink     */
    0,255,0,	 /* lt green */
    255,255,0,	 /* yellow   */
    64,255,144,	 /* aqua     */
    255,255,255};/* white    */

unsigned char rgbawinnew[48] = {
	0,0,0,       /* black    */
	157,9,102,   /* red      */
	42,42,229,   /* dk blue  */
	199,52,255,  /* purple   */
	0,118,26,    /* dk green */
	128,128,128, /* gray     */
	13,161,255,  /* med blue */
	170,170,255, /* lt blue  */
	85,85,0,     /* brown    */
    242,94,0,    /* orange   */
    192,192,192, /* grey     */
    255,137,229, /* pink     */
    56,203,0,    /* lt green */
    213,213,26,  /* yellow   */
    98,246,153,  /* aqua     */
    255,255,254};/* white    */

unsigned char rgbkegs32[48] = {
	0,0,0,		 /* black    */
	221,0,51,	 /* red      */
	0,0,153,	 /* dk blue  */
	221,0,221,	 /* purple   */
	0,119,0,	 /* dk green */
	85,85,85,	 /* gray     */
	34,34,255,	 /* med blue */
	102,170,255, /* lt blue  */
	136,85,34,	 /* brown    */
	255,102,0,	 /* orange   */
	170,170,170, /* grey     */
	255,153,136, /* pink     */
	0,221,0,	 /* lt green */
	255,255,0,	 /* yellow   */
	0,255,153,	 /* aqua     */
	255,255,255};/* white    */

unsigned char rgbciderpress[48] = {
	0,0,0,		 /* black    */
	221,0,51,	 /* red      */
	0,0,153,	 /* dk blue  */
	221,34,221,	 /* purple   */
	0,119,34,	 /* dk green */
	85,85,85,	 /* gray     */
	34,34,255,	 /* med blue */
	102,170,255, /* lt blue  */
	136,85,0,	 /* brown    */
	255,102,0,	 /* orange   */
	170,170,170, /* grey     */
	255,153,136, /* pink     */
	17,221,0,	 /* lt green */
	255,255,0,	 /* yellow   */
	68,255,153,	 /* aqua     */
	255,255,255};/* white    */

unsigned char scanline[960],crt[1],shrpal[32];

#pragma optimize (push,off)
void putpixel320(unsigned x,unsigned y, unsigned char color)
{
    unsigned *src  = (unsigned *)0x3c;
    unsigned *dest = (unsigned *)0x42;
	unsigned pixel, buf = (unsigned) &crt[0];

    /* return if out of range */
    if (x > 319 || y > 199) return;

    pixel = (unsigned) (0x2000 + (y * 160) + (x/2));

	/* move pixel to buffer */
	src[0] = src[1] = pixel;
	dest[0] = buf;
	asm("clc");
	asm("jsr $c311");

	if (x%2 == 1) {
		/* odd pixels in low nibble */
		/* mask value to preserve high nibble */
		crt[0] &= (unsigned char) 240;
		crt[0] |= (unsigned char) color;
	}
	else {
		/* even pixels in high nibble */
		/* mask value to preserve low nibble */
		crt[0] &= (unsigned char) 15;
		crt[0] |= (unsigned char)(color << 4);
	}

	/* move buffer to pixel */
	src[0] = src[1] = buf;
	dest[0] = pixel;
	asm("sec");
	asm("jsr $c311");
}
#pragma optimize (pop)


/* plots pixel in buffer */
void bufferpixel320(unsigned x,unsigned y, unsigned char color)
{
   	unsigned offset = (unsigned) ((y * 160) + (x/2));

    /* return if out of range */
    if (x > 319 || y > 6) return;

	if (x%2 == 1) {
		/* odd pixels in low nibble */
		/* mask value to preserve high nibble */
		scanline[offset] &= (unsigned char) 240;
		scanline[offset] |= (unsigned char) color;
	}
	else {
		/* even pixels in high nibble */
		/* mask value to preserve low nibble */
		scanline[offset] &= (unsigned char) 15;
		scanline[offset] |= (unsigned char)(color << 4);
	}
}

/* plots pixel pairs using siggle-buffering */
#pragma optimize (push,off)
void putpairs320(unsigned x, unsigned y, unsigned pairs, unsigned char color)
{
    unsigned *src  = (unsigned *)0x3c;
    unsigned *dest = (unsigned *)0x42;
	unsigned buf = (unsigned) &scanline[0];

    if (y>199) return;

    /* set pair color */
    if (color < 16)memset((char *)&scanline[0],(char)(color << 4 | color),pairs);

	/* move buffer to shr display */
	src[0] = buf;
	src[1] = buf + pairs - 1;
	dest[0] = (unsigned) (0x2000 + (y * 160) + (x/2));
	asm("sec");
	asm("jsr $c311");
}
#pragma optimize (pop)

#ifdef SETPIXELSONLY
/* simple hline - sets individual pixels - not used */
void hline320(unsigned y,unsigned x1, unsigned x2, unsigned char color)
{
	unsigned temp;

    /* swap co-ordinates if out of order */
    if (x1>x2) {
		temp = x1;
		x1   = x2;
		x2   = temp;
	}

    x2++;
    while(x1 < x2) {
		putpixel320(x1,y,color);
		x1++;
	}
}
#else
/* potentially faster hline using a combination of pixels and pairs */
void hline320(unsigned y,unsigned x1, unsigned x2, unsigned char color)
{
	unsigned temp, pairs;

    /* swap co-ordinates if out of order */
    if (x1>x2) {
		temp = x1;
		x1   = x2;
		x2   = temp;
	}

	if (y > 199 || x1 > 319) return;
	if (x2 > 319) x2=319;

	if (x1 == x2) {
		putpixel320(x1,y,color);
		return;
	}

	if ((x1%2) == 1) {
		putpixel320(x1,y,color);
		x1++;
	}
	if ((x2%2) == 0) {
		putpixel320(x2,y,color);
		x2--;
	}

	pairs = ((x2 + 1) - x1) / 2;

	if (pairs > 0) {
    	putpairs320(x1,y,pairs,color);
	}
}
#endif

/* simple vertical line - must set individual pixels */
void vline320(unsigned x,unsigned y1, unsigned y2, unsigned char color)
{
	unsigned temp;

    /* swap co-ordinates if out of order */
    if (y1>y2) {
		temp = y1;
		y1   = y2;
		y2   = temp;
	}

    y2++;
    while (y1 < y2) {
	    putpixel320(x,y1,color);
		y1++;
	}

}


/* Bresenham Algorithm line drawing routine for SHR mode320 */
void line320(int x1, int y1, int x2, int y2, unsigned char color)
{

	int dx, dy, sx, sy, err, err2;

    /* single pixel */
    if (x1 == x2 && y1 == y2) {
		putpixel320((unsigned)x1,(unsigned)y1,color);
		return;
	}

	/* vertical line */
	if (x1 == x2) {
		vline320((unsigned)x1, (unsigned)y1, (unsigned)y2, color);
		return;

	}

	/* horizontal line */
	if (y1 == y2) {
		hline320((unsigned) y1, (unsigned)x1, (unsigned)x2, color);
		return;
	}

    /* vector */
	if(x1 < x2) {
		dx = x2 - x1;
		sx = 1;
	}
	else {
		sx = -1;
		dx = x1 - x2;
	}

	if(y1 < y2) {
		sy = 1;
		dy = y2 - y1;
	}
	else {
		sy = -1;
		dy = y1 - y2;
	}

	err = dx-dy;

	for (;;) {
		putpixel320((unsigned)x1,(unsigned)y1,color);

		if(x1 == x2 && y1 == y2)break;

		err2 = err*2;

		if(err2 > (0-dy)) {
			 err = err - dy;
			 x1 = x1 + sx;
		}

		if(err2 <  dx) {
			 err = err + dx;
			 y1 = y1 + sy;
		}
	}
}


/* Aztec C65 circle algorithm, taken from the HGR mode Aztec C65 3.2b version,
   and originally written circa 1982-83, then modified for SHR */
void circle320(int cx,int cy,int r, unsigned char color)
{

    int x,y,a,b,c,d,f;

    x=r; y=0; b=1; f=0;

    a=(-2)*x+1;

point:

	c=   (int) x;
	d=   (int) y;

    putpixel320((unsigned)(c+cx), (unsigned)(y+cy), color);
	putpixel320((unsigned)(d+cx), (unsigned)(x+cy), color);
    putpixel320((unsigned)(-d+cx),(unsigned)(x+cy), color);
    putpixel320((unsigned)(-c+cx),(unsigned)(y+cy), color);
    putpixel320((unsigned)(-c+cx),(unsigned)(-y+cy),color);
	putpixel320((unsigned)(-d+cx),(unsigned)(-x+cy),color);
    putpixel320((unsigned)(d+cx), (unsigned)(-x+cy),color);
    putpixel320((unsigned)(c+cx), (unsigned)(-y+cy),color);

    if(b>= -a)
        goto fin;

    y+=1; f+=b;
    b+=2;

    if(f>r)
    {
        f+=a;
        a+=2;
        x-=1;
    }
    goto point;

fin:;

}

/* filled disk */
void disk320(int cx,int cy,int r, unsigned char color)
{

    int x,y,a,b,c,d,f;

    x=r; y=0; b=1; f=0;

    a=(-2)*x+1;

point:

	c=   (int) x;
	d=   (int) y;

    hline320((unsigned)(y+cy),(unsigned)(-c+cx),(unsigned)(c+cx),color);
    hline320((unsigned)(x+cy),(unsigned)(-d+cx),(unsigned)(d+cx),color);
    hline320((unsigned)(-y+cy),(unsigned)(-c+cx),(unsigned)(c+cx),color);
    hline320((unsigned)(-x+cy),(unsigned)(-d+cx),(unsigned)(d+cx),color);

    if(b>= -a)
        goto fin;

    y+=1; f+=b;
    b+=2;

    if(f>r)
    {
        f+=a;
        a+=2;
        x-=1;
    }
    goto point;

fin:;

}

/* filled or unfilled boxes */
void box320(int x1, int y1, int x2, int y2, unsigned char color, unsigned char fillcolor)
{
	hline320((unsigned)y1,(unsigned)x1,(unsigned)x2,color);
	y1++;
	while (y1 < y2) {
		putpixel320((unsigned)x1,(unsigned)y1,color);
		if (fillcolor < 16) hline320((unsigned)y1,(unsigned)x1+1,(unsigned)x2-1,fillcolor);
		putpixel320((unsigned)x2,(unsigned)y1,color);
		y1++;
	}
	hline320((unsigned)y2,(unsigned)x1,(unsigned)x2,color);
}

/* color block box frame used in pseudo-animation demo */
/* not really a core routine but stuck here anyway */
unsigned char candyframe320(unsigned char color)
{

	/* uses an 8 x 8 cell - 320 x 200 pixels = 40 x 25 cells*/
	unsigned x,y,i,j;
	unsigned char ch = color;

    for (i = 0,x = 0;i<40;i++,x+=8) {
	    for (y = 0;y<8;y++) putpairs320(x,y,4,color);
	    color++;
	    if (color > 15)color = 1;
	}
	for (i = 1, y=8; i < 24;i++,y+=8) {
		for (j = 0;j<8;j++) putpairs320(312,y+j,4,color);
		color++;
		if (color > 15)color = 1;
	}
    for (i = 0,x = 312;i<40;i++,x-=8) {
	    for (y = 0;y<8;y++) putpairs320(x,192+y,4,color);
	    color++;
	    if (color > 15)color = 1;
	}
	for (i = 23, y=184; i > 0;i--,y-=8) {
		for (j = 0;j<8;j++) putpairs320(0,y+j,4,color);
		color++;
		if (color > 15)color = 1;
	}

    ch++;
	if (ch > 15)ch = 1;
	return ch;
}

/* clear the SHR screen to color
   probably no good if unknown palettes are used
   mostly for single-palette pixel graphics and the like */
void clear320(unsigned char color)
{
	unsigned char paircolor = (unsigned char) (color << 4 | color);
	unsigned y;
	unsigned src1 = (unsigned) &scanline[0], src2 = src1+159;

	/* put pixel pair array on SHR display */
	/* fill scanline buffer with color pairs */
    memset((char *)&scanline[0],paircolor,160);
    /* wipe-down 200 scan lines with color */
    for (y=0;y<200;y++) {
		maintoaux(src1,src2,(unsigned) (0x2000 + (y * 160)));

	}
}

/* sets a 12 bit color palette line from an array of 24 bit color values */
/* rgbpal is a 48 byte character array of 16 - r,g,b values */
/* palidx is the SHR palette number in the range of 0-15 */
void setpalette(unsigned char *rgbpal,int palidx)
{
	unsigned char r,g,b;
	int i,j,k;
	unsigned src1 = (unsigned)&shrpal[0], src2 = src1 + 31;

	/* build 12 bit palette line of $0RGB color entries
	   from 24 bit r,g,b values */
	for (i=0,j=0,k=0;i<16;i++,j+=3,k+=2) {
		r = rgbpal[j]   >> 4;
		g = rgbpal[j+1] >> 4;
		b = rgbpal[j+2] >> 4;
		shrpal[k] = (unsigned char)((g << 4) | b);
		shrpal[k+1] = r;
	}

    /* move palette line to palette */
    maintoaux(src1, src2,(unsigned)(0x9e00 + (palidx * 32)));

}

/* mono-spaced "tom thumb" 4 x 6 font */
/* using a byte map to gain a little speed at the expense of memory */
/* a bitmap could have been encoded into nibbles of 3 bytes per character
   rather than the 18 bytes per character that I am using
   but the trade-off in the speed in unmasking would have slowed this down */
void plotthumb320(unsigned char ch, unsigned x, unsigned y,
               unsigned char fg, unsigned char bg)
{
	unsigned offset, x1, x2=x+3, y2=y+6;
	unsigned char byte;

	if (ch < 33 || ch > 127) ch = 0;
	else ch -=32;

	if (ch == 0 && bg > 15) return;

    /* each of the 96 characters is encoded into 18 bytes */
	offset = (18 * ch);

    while (y < y2) {
		for (x1 = x; x1 < x2; x1++) {
		   if (x1 > 319) {
			   offset++;
			   continue;
		   }

		   byte = tomthumb[offset++];

		   if (byte == 0) {
			   if (bg > 15) continue;
			   putpixel320(x1,y,bg);
		   }
		   else {
			   if (fg > 15) continue;
			   putpixel320(x1,y,fg);
	   	   }
		}
		/* if background color is being used then a trailing pixel is required
		   between characters */
		if (bg < 16 && x2 < 320)putpixel320(x2,y,bg);

		if (y++ > 199) break;
	}

}

/* normally spaced 4 x 6 font */
/* using character plotting function plotthumb() (above) */
void thumb320(char *str,unsigned x, unsigned y,
              unsigned char fg,unsigned char bg, unsigned char justify)
{
  int target;
  unsigned char ch;

  if (justify == 'M' || justify == 'm') {
	 target = strlen(str);
	 x-= ((target * 4) /2);
  }

  while ((ch = *str++) != 0) {
	 plotthumb320(ch,x,y,fg,bg);
	 x+=4;
  }
}

/* functions to plot a font line into buffered memory */
/* faster but not as "fluid" as the font routines above */

void tinychar(unsigned char ch, unsigned x,
           unsigned char fg, unsigned char bg)
{
	unsigned offset, x1, x2=x+3, y, y2=6;
	unsigned char byte;

	if (ch < 33 || ch > 127) ch = 0;
	else ch -=32;

	if (ch == 0 && bg > 15) return;

    /* each of the 96 characters is encoded into 18 bytes */
	offset = (18 * ch);

    for(y = 0; y < 6; y++) {
		for (x1 = x; x1 < x2; x1++) {
		   if (x1 > 319) {
			   offset++;
			   continue;
		   }

		   byte = tomthumb[offset++];

		   if (byte == 0) {
			   if (bg > 15) continue;
			   bufferpixel320(x1,y,bg);
		   }
		   else {
			   if (fg > 15) continue;
			   bufferpixel320(x1,y,fg);
	   	   }
		}
		/* if background color is being used then a trailing pixel is required
		   between characters */
		if (bg < 16 && x2 < 320)bufferpixel320(x2,y,bg);

	}

}


void tinyplots(char *str,unsigned x, unsigned y,
              unsigned char fg,unsigned char bg, unsigned char justify)
{
  int target;
  unsigned char ch;
  unsigned src1, buf = (unsigned)&scanline[0];


  if (justify == 'M' || justify == 'm') {
	 target = strlen(str);
	 if (target == 0) return;
	 x-= ((target * 4) /2);

  }

  if (x > 319) x = 0;
  if (y > 194) y = 0;

  /* copy 6 scanlines to scanline buffer */
  src1 = (unsigned) 0x2000 + (y * 160);
  auxtomain(src1,src1+959,buf);

  /* plot 1 font line in 6 buffered scanlines */
  while ((ch = *str++) != 0) {
	 tinychar(ch,x,fg,bg);
	 x+=4;
  }

  /* copy updated lines back to SHR memory */
  maintoaux(buf,buf+959,src1);

}
